#include "rebocap_ws_sdk_cpp.h"

namespace rebocap {
RebocapWsSdk::RebocapWsSdk(CoordinateSpaceType coordinateSpaceType)
    : handle_(rebocap_ws_sdk_new(coordinateSpaceType), [](void *ptr) { rebocap_ws_sdk_release(ptr); }) {
  rebocap_ws_sdk_set_pose_msg_callback(handle_.get(), this, RebocapWsSdk::pose_msg_callback);
  rebocap_ws_sdk_set_exception_close_callback(handle_.get(), this, RebocapWsSdk::exception_close_callback);
}

int RebocapWsSdk::Open(uint32_t port, std::string_view name, int64_t uid) {
  return rebocap_ws_sdk_open(handle_.get(), port, name.data(), name.size(), uid);
}

void RebocapWsSdk::Close() { rebocap_ws_sdk_close(handle_.get()); }

void RebocapWsSdk::SetPoseMsgCallback(std::function<void(const struct QuatMsg *, RebocapWsSdk *)> &&func) {
  pose_msg_callback_ = std::move(func);
}

void RebocapWsSdk::SetExceptionCloseCallback(std::function<void(RebocapWsSdk *)> &&func) { exception_close_callback_ = std::move(func); }

void RebocapWsSdk::pose_msg_callback(const struct QuatMsg *msg, void *extra) {
  auto *that = reinterpret_cast<RebocapWsSdk *>(extra);
  that->PoseMsgCallback(msg);
  if (that->pose_msg_callback_) {
    that->pose_msg_callback_(msg, that);
  }
}

void RebocapWsSdk::exception_close_callback(void *extra) {
  auto *that = reinterpret_cast<RebocapWsSdk *>(extra);
  that->ExceptionCloseCallback();
  if (that->exception_close_callback_) {
    that->exception_close_callback_(that);
  }
}

int RebocapWsSdk::GetLastMsg(QuatMsg *msg) { return rebocap_ws_sdk_get_last_msg(handle_.get(), msg); }
}  // namespace rebocap